import { getKysely } from "./kysely";
import { jsonArrayFrom } from "kysely/helpers/sqlite";
import type { Tag, TagAccessor, TagCount } from "../model/tag";
import type { DBTagContentRelation } from "./doc";

class SqliteTagAccessor implements TagAccessor {

	constructor(private kysely = getKysely()) {
	}
	async getAllTagCount(): Promise<TagCount[]> {
		const result = await this.kysely
			.selectFrom("doc_tag_relation")
			.select("tag_name")
			.select(qb => qb.fn.count<number>("doc_id").as("occurs"))
			.groupBy("tag_name")
			.execute();
		return result;
	}
	async getAllTagList(): Promise<Tag[]> {
		return (await this.kysely.selectFrom("tags")
			.selectAll()
			.execute()
			).map((x) => ({
				name: x.name,
				description: x.description ?? undefined,
			}));
	}
	async getTagByName(name: string) {
		const result = await this.kysely
			.selectFrom("tags")
			.selectAll()
			.where("name", "=", name)
			.executeTakeFirst();
		if (result === undefined) {
			return undefined;
		}
		return {
			name: result.name,
			description: result.description ?? undefined,
		};
	}
	async addTag(tag: Tag) {
		const result = await this.kysely.insertInto("tags")
			.values([tag])
			.onConflict((oc) => oc.doNothing())
			.executeTakeFirst();
		return (result.numInsertedOrUpdatedRows ?? 0n) > 0;
	}
	async delTag(name: string) {
		const result = await this.kysely.deleteFrom("tags")
			.where("name", "=", name)
			.executeTakeFirst();
		return (result.numDeletedRows ?? 0n) > 0;
	}
	async updateTag(name: string, desc: string) {
		const result = await this.kysely.updateTable("tags")
			.set({ description: desc })
			.where("name", "=", name)
			.executeTakeFirst();
		return (result.numUpdatedRows ?? 0n) > 0;
	}
}
export const createSqliteTagController = (kysely = getKysely()): TagAccessor => {
	return new SqliteTagAccessor(kysely);
};
